-----------------------------------------------------------------------------------------------
-- Client Lua Script for PVPTab
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ICComm"

-----------------------------------------------------------------------------------------------
-- PVPTab Module Definition
-----------------------------------------------------------------------------------------------
local PVPTab = {} 

local tClass = 	{ 
	[GameLib.CodeEnumClass.Warrior]      = { strName = Apollo.GetString("ClassWarrior"), 	    strLogo = "CRB_Raid:sprRaid_Icon_Class_Warrior" },		
	[GameLib.CodeEnumClass.Engineer]     = { strName = Apollo.GetString("ClassEngineer"), 	 	strLogo = "CRB_Raid:sprRaid_Icon_Class_Engineer" },		
	[GameLib.CodeEnumClass.Esper]        = { strName = Apollo.GetString("ClassESPER"), 		 	strLogo = "CRB_Raid:sprRaid_Icon_Class_Esper" },		
	[GameLib.CodeEnumClass.Medic]        = { strName = Apollo.GetString("ClassMedic"), 		 	strLogo = "CRB_Raid:sprRaid_Icon_Class_Medic" },		
	[GameLib.CodeEnumClass.Stalker]      = { strName = Apollo.GetString("ClassStalker"), 	    strLogo = "CRB_Raid:sprRaid_Icon_Class_Stalker" },		
	[GameLib.CodeEnumClass.Spellslinger] = { strName = Apollo.GetString("ClassSpellslinger"),  	strLogo = "CRB_Raid:sprRaid_Icon_Class_Spellslinger" },	
}
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function PVPTab:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	-- misc vars
	self.liveStatistics = nil
	self.peEvent = nil
	self.players = {}
	self.units = {}
	self.orderId = 1
	
	self.lastUpdate = os.clock()
	
	self.ratingShare = nil
	self.ratingShareChannel = nil
	self.ratingShareTimer = nil
	
    return o
end

function PVPTab:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- PVPTab OnLoad
-----------------------------------------------------------------------------------------------
function PVPTab:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("PVPTab.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
	Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
end

-----------------------------------------------------------------------------------------------
-- PVPTab OnDocLoaded
-----------------------------------------------------------------------------------------------
function PVPTab:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "PVPTabForm", nil, self)
		self.wndOrdering = Apollo.LoadForm(self.xmlDoc, "orderingForm", nil, self)
		self.wndSettings = Apollo.LoadForm(self.xmlDoc, "settingsForm", nil, self)
		
		if self.wndMain == nil or self.wndOrdering == nil or self.wndSettings == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		-- Hide PVPTAB
	    self.wndMain:Show(false, true)
		self.wndOrdering:Show(false, true)
		self.wndSettings:Show(false, true)
	
		-- Listen events
		Apollo.RegisterEventHandler("PublicEventStart", "OnPublicEventStart", self)
		Apollo.RegisterEventHandler("PublicEventLiveStatsUpdate", "OnPublicEventStart", self)
		Apollo.RegisterEventHandler("SystemKeyDown", "OnSystemKeyDown", self)
		Apollo.RegisterEventHandler("MatchEntered", "OnMatchEntered", self)	
		Apollo.RegisterEventHandler("MatchExited", "OnMatchExited", self)
		Apollo.RegisterEventHandler("WindowSizeChanged", "OnWindowSizeChanged", self)
		Apollo.RegisterEventHandler("Group_Remove", "OnGroupRemove", self)
		Apollo.RegisterEventHandler("PVPMatchStateUpdated", "OnPVPMatchStateUpdated", self)
		
		-- Events contest afk system
		Apollo.RegisterEventHandler("UnitEvaded", "ContestAfk", self)
		Apollo.RegisterEventHandler("UnitEnteredCombat", "ContestAfk", self)

		self:CheckEvents()
		self:OnWindowSizeChanged()
	end
end

function PVPTab:OnSave(eLevel)	
	return {
		["keyBind"] = self.keyBind,
		["enableRS"] = self.enableRS
	}
end

function PVPTab:OnRestore(eLevel, tData)
	self.keyBind = tData.keyBind or 9
	self.enableRS = tData.enableRS or true
end

function PVPTab:LoadSettingsUI()
	self.wndSettings:FindChild("checkRS"):SetCheck(self.enableRS)
end

function PVPTab:SetSettingsUI()
	self.enableRS = self.wndSettings:FindChild("checkRS"):IsChecked()
end
	
-- When the match state is change (prep, progress, finished)
function PVPTab:OnPVPMatchStateUpdated()
	local statut = MatchingGame:GetPVPMatchState().eState
	if statut == 1 then
		-- Team switch in juresang
		if self.peEvent then
			if self.peEvent:GetEventType() == 4 then
				-- Redefine team
				self.wndMain:FindChild("teamB"):SetData(nil)
				self.wndMain:FindChild("teamA"):SetData(nil)
			end
		end
	end
end

-- Check all current existing events
function PVPTab:CheckEvents()
	local events = PublicEventsLib:GetActivePublicEventList()
	for key, event in pairs(events) do
		self:OnPublicEventStart(event)
	end
end

-- When a player leave your group, auto afk
function PVPTab:OnGroupRemove(playerName, eReason)
	if MatchingGame.IsInMatchingGame() or MatchingGame.IsInPVPGame() then 
		-- If the match is in progress
		if self:IsMatchFinish() == false then
			if eReason == 5 then
				if self.players[playerName] then
					-- Define afk
					self:PlayerAfk(self.players[playerName], true)
				end
			end
		end
	end
end

-- When we receive action that can contest AFK system
function PVPTab:ContestAfk(unit)
	local player = self.players[unit:GetName()]
	
	if player then
		-- If player is already afk
		if player.afk then
			self:PlayerAfk(player, false)
		else
			player.afkCounter = 0
		end
	end
end

-- When player start a pvp instance, we reset PVPTAB
function PVPTab:OnMatchEntered()
	self:ResetAll()
end

-- When player exist a match (intentionnal or not)
function PVPTab:OnMatchExited()
	self.wndMain:Show(false, true)
	self.wndOrdering:Show(false, true)
	-- Disable share rating
	self.ratingShare = nil
end

-- When reset all for new matching game
function PVPTab:ResetAll()
	-- Reset all stats
	self.liveStatistics = nil
	self.peEvent = nil
	self.players = {}
	self.units = {}
	self.orderId = 1
	self.lastUpdate = os.clock()
	self.ratingShare = nil
	self.ratingShareChannel = nil
	self.ratingShareTimer = nil
	-- Reset all UI
	self.wndMain:FindChild("teamA"):DestroyChildren()
	self.wndMain:FindChild("teamA"):SetData(nil)
	self.wndMain:FindChild("teamAText"):SetText("Loading...")
	self.wndMain:FindChild("teamAIcon"):SetSprite("CRB_Anim_Spinner:sprAnim_SpinnerSmall")
	self.wndMain:FindChild("teamAKDA"):SetText("0/0/0")
	self.wndMain:FindChild("damageDealtTA"):SetText("Loading...")
	self.wndMain:FindChild("healDoneTA"):SetText("Loading...")
	self.wndMain:FindChild("teamB"):DestroyChildren()
	self.wndMain:FindChild("teamB"):SetData(nil)
	self.wndMain:FindChild("teamBText"):SetText("Loading...")
	self.wndMain:FindChild("teamBIcon"):SetSprite("CRB_Anim_Spinner:sprAnim_SpinnerSmall")
	self.wndMain:FindChild("teamBKDA"):SetText("0/0/0")
	self.wndMain:FindChild("damageDealtTB"):SetText("Loading...")
	self.wndMain:FindChild("healDoneTB"):SetText("Loading...")
	-- Research event
	self:CheckEvents()
end

-- When the wildstar window is resized, recenter it
function PVPTab:OnWindowSizeChanged()
	local wx = Apollo.GetDisplaySize().nWidth * 0.5
	local wy = Apollo.GetDisplaySize().nHeight * 0.4

	-- Main Form
	local x = wx - (self.wndMain:GetWidth() * 0.5)
	local y = wy - (self.wndMain:GetHeight() * 0.5)
	self.wndMain:SetAnchorOffsets(x, y, self.wndMain:GetWidth() + x, self.wndMain:GetHeight() + y)
	
	-- Ordering Form
	local x = wx - (self.wndOrdering:GetWidth() * 0.5)
	local y = wy - (self.wndOrdering:GetHeight() * 0.5)
	self.wndOrdering:SetAnchorOffsets(x, y, self.wndOrdering:GetWidth() + x, self.wndOrdering:GetHeight() + y)
	
	-- Settings Form
	local x = wx - (self.wndSettings:GetWidth() * 0.5)
	local y = wy - (self.wndSettings:GetHeight() * 0.5)
	self.wndSettings:SetAnchorOffsets(x, y, self.wndSettings:GetWidth() + x, self.wndSettings:GetHeight() + y)
end

-- When player press key a key
function PVPTab:OnSystemKeyDown(iKey, test)
	local canShowIt = false

	-- If TAB pressed
	if iKey == 9 then

		if MatchingGame.IsInMatchingGame() or MatchingGame.IsInPVPGame() then 
			canShowIt = true
		elseif self.liveStatistics then
			-- If we have stats from match exited
			canShowIt = true
		end
		
		canShowIt = true

		if canShowIt then
			-- If the window is visible
			if not self.wndMain:IsVisible() then
				self.wndMain:Invoke()
			else 
				-- if the window is not shown
				self.wndMain:Close()
				self.wndOrdering:Close()
				self.wndSettings:Close()
			end
		end
	end
end

-- Return if we are in a preparation phase of current matching game
function PVPTab:IsOnPreparePhase()
	if MatchingGame:GetPVPMatchState().eState == 1 then
		return true
	else
		return false
	end
end

-- Return if a matchinggame is finish or not
function PVPTab:IsMatchFinish()
	if MatchingGame:GetPVPMatchState().eState == 3 then
		return true
	else
		return false
	end
end

-- When matchingame event start or is updated
function PVPTab:OnPublicEventStart(peEvent)
	self.lastUpdate = os.clock()
	if MatchingGame.IsInMatchingGame() or MatchingGame.IsInPVPGame() then
		if not self.ratingShare then
			self:StartRatingShare()
		end
		
		if peEvent:HasLiveStats() then
			self.peEvent = peEvent
			self.liveStatistics = peEvent:GetLiveStats()
			self:Update()
			
			-- Size interface relative to player count
			if self.peEvent:GetEventType() == 4 or self.peEvent:GetEventType() == 3 or self.peEvent:GetEventType() == 1 then
				-- In walatiki and juresang and arena
				self:defineInterfaceForPlayers(10)
			else
				-- Daggerstone
				self:defineInterfaceForPlayers(15)
			end
		end
	end
end

-- Define height of interface for player count
function PVPTab:defineInterfaceForPlayers(playerCount)

	if playerCount == 10 then
	
		self.wndMain:SetAnchorOffsets(0, 0, 819, 487)
		self.wndMain:FindChild("OrderBtn"):SetAnchorOffsets(553, 457, 679, 479)
		self.wndMain:FindChild("settingsBtn"):SetAnchorOffsets(684, 457, 810, 479)
		self.wndMain:FindChild("OrderText"):SetAnchorOffsets(448, 462, 571, 475)
		self.wndMain:FindChild("sep"):SetAnchorOffsets(410, 50, 411, 460)
		self.wndMain:FindChild("teamA"):SetAnchorOffsets(6, 51, 410, 449)
		self.wndMain:FindChild("teamB"):SetAnchorOffsets(416, 51, 820, 452)
		
	elseif playerCount == 15 then
	
		self.wndMain:SetAnchorOffsets(0, 0, 819, 694)
		self.wndMain:FindChild("OrderBtn"):SetAnchorOffsets(553, 659, 679, 681)
		self.wndMain:FindChild("settingsBtn"):SetAnchorOffsets(684, 659, 810, 681)
		self.wndMain:FindChild("OrderText"):SetAnchorOffsets(448, 663, 571, 676)
		self.wndMain:FindChild("sep"):SetAnchorOffsets(410, 50, 411, 652)
		self.wndMain:FindChild("teamA"):SetAnchorOffsets(6, 51, 410, 650)
		self.wndMain:FindChild("teamB"):SetAnchorOffsets(416, 51, 820, 650)

	end
	
	self:OnWindowSizeChanged()
end

-- When a unit is received in area player
function PVPTab:OnUnitCreated(unit)
	-- Only players units
	if unit:GetType() == "Player" then
		-- Store unit
		self.units[unit:GetName()] = unit
		
		-- Define unit for existants players
		for key, player in pairs(self.players) do
			local playerName = unit:GetName()
			if playerName == player.name then
				self:AttachUnitToPlayer(playerName, unit)
			end
		end
		
		self:ContestAfk(unit)
	end
end

-- When a unit is destroyed in area player
function PVPTab:OnUnitDestroyed(unit) 
	-- Only players units
	if unit:GetType() == "Player" then
		-- Remove from units
		self.units[unit:GetName()] = nil
		if self.players[unit:GetName()] then
			-- If this unit has player attached
			self.players[unit:GetName()].unitAttached = false
		end
	end
end

-- When we attach unit to a player for set his playerFace and define his role if teammate
function PVPTab:AttachUnitToPlayer(playerName, playerUnit)
	local player = self.players[playerName]
	
	-- If this process is not already executed
	if not player.unitAttached then
		
		-- Define his unit
		player.unit = playerUnit
		
		-- Define his face
		player.interface:FindChild("playerFace"):SetCostume(playerUnit)
		player.interface:FindChild("playerFace"):Show(true, true)
		player.interface:FindChild("playerLoading"):Show(false, true)

		-- Define his role if in my team
		local AP = playerUnit:GetAssaultPower() or 0
		if AP ~= 0 then
			-- Team mate
			self:DefineTeamMateRole(player)
			player.enemy = false
		else
			-- Enemy
			player.enemy = true
		end
		
		-- Its ok
		player.unitAttached = true
	end
end

-- Order players by the selectionned id
function PVPTab:OrderPlayers(OrderId)

	-- If need ordering
	if OrderId then
		for key, player in pairs(self.players) do
			if player.interface then					
				if OrderId == 1 then
					-- Damage dealt
					player.interface:SetData(player.damage)
				elseif OrderId == 2 then
					-- Heal done
					player.interface:SetData(player.heal)
				elseif OrderId == 3 then
					-- Most kills
					player.interface:SetData(player.kills)		
				elseif OrderId == 4 then
					-- Most deaths
					player.interface:SetData(player.deaths)		
				elseif OrderId == 5 then
					-- Most assists
					player.interface:SetData(player.assists)		
				end
			end
		end
	end

	-- ReOrder players
	self.wndMain:FindChild("teamA"):ArrangeChildrenVert(0, function(a,b) return a:GetData() > b:GetData() end)
	self.wndMain:FindChild("teamB"):ArrangeChildrenVert(0, function(a,b) return a:GetData() > b:GetData() end)
end

-- Define both teams of current instance matching game
function PVPTab:DefineTeamContainers(teamName, isMyTeam)
	local teamAData = self.wndMain:FindChild("teamA"):GetData()
	local teamBData = self.wndMain:FindChild("teamB"):GetData()
	
	-- if teamAData if not define
	if teamAData == nil and isMyTeam then
		self.wndMain:FindChild("teamA"):SetData(teamName)
		self.wndMain:FindChild("teamAText"):SetText(teamName)
	if string.match(teamName, "rouge") or string.match(teamName, "red") or teamName == "Attaquant" or teamName == "Attacker" then
			self.wndMain:FindChild("teamAIcon"):SetSprite("CRB_InterfaceMenuList:spr_InterfaceMenuList_RedFlagStretch")
			self.wndMain:FindChild("teamAText"):SetTextColor(ApolloColor.new("ChatPvP"))
		else 
			self.wndMain:FindChild("teamAIcon"):SetSprite("CRB_InterfaceMenuList:spr_InterfaceMenuList_BlueFlagStretch")
			self.wndMain:FindChild("teamAText"):SetTextColor(ApolloColor.new("xkcdCerulean"))
		end
	else 
		-- If teamBData if not define and teamAData is different than actual player team name
		if teamBData == nil and teamAData ~= teamName then
			self.wndMain:FindChild("teamB"):SetData(teamName)
			self.wndMain:FindChild("teamBText"):SetText(teamName)		
			if not string.match(teamName, "rouge") and not string.match(teamName, "red") and teamName ~= "Attaquant" and teamName ~= "Attacker" then
				self.wndMain:FindChild("teamBIcon"):SetSprite("CRB_InterfaceMenuList:spr_InterfaceMenuList_BlueFlagStretch")
				self.wndMain:FindChild("teamBText"):SetTextColor(ApolloColor.new("xkcdCerulean"))
			else 
				self.wndMain:FindChild("teamBIcon"):SetSprite("CRB_InterfaceMenuList:spr_InterfaceMenuList_RedFlagStretch")
				self.wndMain:FindChild("teamBText"):SetTextColor(ApolloColor.new("ChatPvP"))
			end
		end
	end
end

-- Make a new player container in PVPTab
function PVPTab:MakePlayerContainer(playerInfo)
	-- Init it
	local key = playerInfo.strName
	self.players[key] = {}
	self.players[key].name = playerInfo.strName
	self.players[key].afkCounter = 0
	self.players[key].afk = false
	self.players[key].unitAttached = false
	self.players[key].statsDefined = false
	self:UpdatePlayerStats(playerInfo)
	self.players[key].classId = playerInfo.eClass
	
	-- Define Team Container for this player
	local teamAData = self.wndMain:FindChild("teamA"):GetData()
	local teamContainer = self.wndMain:FindChild("teamB")
	if teamAData == playerInfo.strTeamName then
		teamContainer = self.wndMain:FindChild("teamA")
	end

	-- Create his interface
	self.players[key].interface = Apollo.LoadForm(self.xmlDoc, "playerContainer", teamContainer, self)
	self.players[key].interface:SetData(playerInfo.nDamage)
	self:UpdatePlayerContainer(playerInfo)
	self:OrderPlayers()
	
	-- player class icon and his tooltip
	self.players[key].interface:FindChild("playerClassIcon"):SetSprite(tClass[playerInfo.eClass].strLogo)
	self.players[key].interface:FindChild("playerClassIcon"):SetTooltip(tClass[playerInfo.eClass].strName)
	
	-- player name, substringed if needed
	local name = playerInfo.strName
	if string.len(name) >= 18 then
		name = string.sub(playerInfo.strName, 1, 18)..".."
		self.players[key].interface:FindChild("playerName"):SetTooltip(playerInfo.strName)
	end
	self.players[key].interface:FindChild("playerName"):SetText(name)
	
	-- Check if there is existant unit for this playerName
	if self.units[key] then
		self:AttachUnitToPlayer(playerInfo.strName, self.units[key])
	elseif playerInfo.unitParticipant then
		self:AttachUnitToPlayer(playerInfo.strName, playerInfo.unitParticipant)
	end
end

-- Player has new statut for afk
function PVPTab:PlayerAfk(player, is) 
	if is then
		player.afk = true
		player.interface:Show(false, true)
	else
		player.afk = false
		player.interface:Show(true, true)
		self:OrderPlayers(self.orderId)
	end
end

-- Store in vars new stats and define afk or not
function PVPTab:UpdatePlayerStats(playerInfo)
	local player = self.players[playerInfo.strName]
	
	-- Not in lobby waiting the game to start or the end of a match
	if self:IsOnPreparePhase() == false and self:IsMatchFinish() == false then
		-- No stats change since x updates
		if playerInfo.nDamage == player.damage and playerInfo.nHealed == player.heal and player.healReceived == playerInfo.nHealingReceived and player.damageReceived == playerInfo.nDamageReceived then
			-- Player afk while 1 updates events
			player.afkCounter = player.afkCounter + 1
		else
			-- The player seem to be not afk
			player.afkCounter = 0
		
			-- No more afk
			if player.afk then
				self:PlayerAfk(player, false)
			end
		end
		
		-- He is here if we can read his unit
		if player.unitAttached then
			player.afkCounter = 0
		end

		-- If the player is afk since 50 secs (10 updates, 1 update = 5 secs)
		if player.afkCounter >= 10 and player.afk == false then
			-- If there is no stats since 6 updates, considere this player has leaver/afk
			self:PlayerAfk(player, true)
		end
	end

	player.damage = playerInfo.nDamage or 0
	player.heal = playerInfo.nHealed or 0
	
	player.kills = playerInfo.nKills or 0
	player.deaths = playerInfo.nDeaths or 0
	player.assists = playerInfo.nAssists or 0

	player.damageReceived = playerInfo.nDamageReceived or 0
	player.healReceived = playerInfo.nHealingReceived or 0

	player.statsDefined = true

	self:OrderPlayers(self.orderId)
end

-- When we update player container
function PVPTab:UpdatePlayerContainer(playerInfo)
	self:UpdatePlayerStats(playerInfo)	

	-- clear badges
	self.players[playerInfo.strName].interface:FindChild("playerBadges"):DestroyChildren()
		
	-- player KDA
	self.players[playerInfo.strName].interface:FindChild("KDA"):SetText(playerInfo.nKills.."/"..playerInfo.nDeaths.."/"..playerInfo.nAssists)
	
	-- player damage
	local damage = round(playerInfo.nDamage/1000)
	if damage == 0 then
		self.players[playerInfo.strName].interface:FindChild("damageDealt"):SetText("NO DPS")
	else
		self.players[playerInfo.strName].interface:FindChild("damageDealt"):SetText(damage.."K DPS")
	end
	
	-- player heal
	local heal = round(playerInfo.nHealed/1000)
	if heal == 0 then
		self.players[playerInfo.strName].interface:FindChild("healDone"):SetText("NO HEAL")
	else
		self.players[playerInfo.strName].interface:FindChild("healDone"):SetText(heal.."K HEAL")
	end
end

-- When we update stats of a teams (kda, heal, dps)
function PVPTab:UpdateTeamStats(team)
	if team.strTeamName == self.wndMain:FindChild("teamAText"):GetText() then
		self.wndMain:FindChild("teamAKDA"):SetText(team.nKills.."/"..team.nDeaths.."/"..team.nAssists)
		local damage = round(team.nDamage/1000)
		if damage == 0 then
			self.wndMain:FindChild("damageDealtTA"):SetText("NO DPS")
		else
			self.wndMain:FindChild("damageDealtTA"):SetText(damage.."K DPS")
		end
		
		local heal = round(team.nHealed/1000)
		if heal == 0 then
			self.wndMain:FindChild("healDoneTA"):SetText("NO HEAL")
		else 
			self.wndMain:FindChild("healDoneTA"):SetText(heal.."K HEAL")
		end
	else
		self.wndMain:FindChild("teamBKDA"):SetText(team.nKills.."/"..team.nDeaths.."/"..team.nAssists)
		local damage = round(team.nDamage/1000)
		if damage == 0 then
			self.wndMain:FindChild("damageDealtTB"):SetText("NO DPS")
		else
			self.wndMain:FindChild("damageDealtTB"):SetText(damage.."K DPS")
		end
		
		local heal = round(team.nHealed/1000)
		if heal == 0 then
			self.wndMain:FindChild("healDoneTB"):SetText("NO HEAL")
		else 
			self.wndMain:FindChild("healDoneTB"):SetText(heal.."K HEAL")
		end
	end
end

-- When we start ratingShare
function PVPTab:StartRatingShare()
	-- Make channel name
	local channelName = "PVPTab"..MatchingGame:GetCurrentMatchingGame():GetGameId()
	self.ratingShareChannel = channelName
	
	-- Make it !
	self.ratingShare = ICCommLib.JoinChannel(channelName, ICCommLib.CodeEnumICCommChannelType.Global)

	-- Configure it !
	if self.ratingShare:IsReady() then
    	self.ratingShare:SetReceivedMessageFunction("OnRatingReceived", self)
    	self.ratingShare:SetSendMessageResultFunction("OnRatingSendConfirmation", self)
	else 
		self.ratingShare = nil
	end
end

-- When a player send his rating to me
function PVPTab:OnRatingReceived(channel, strMessage, strPlayerName)

	-- Define players object
	local player = self.players[strPlayerName]

	-- If players exist in this match
	if player then

		-- Parse message
		local data = string.split(strMessage, ".")

		-- Complete data ?
		if table.getn(data) == 3 then
			
			-- Set his rating vars
			player.rating = data[1]
			player.wins = data[2]
			player.losses = data[3]

		end
	end
end

-- When rating send is confirmed
function PVPTab:OnRatingSendConfirmation(iccomm, eResult, idMessage)
	
end

-- Split a string
function string:split(sep)
    local sep, fields = sep or ":", {}
    local pattern = string.format("([^%s]+)", sep)
    self:gsub(pattern, function(c) fields[#fields+1] = c end)
    return fields
end

-- Send my own rating on ratingShare
function PVPTab:SendMyRating()
	if MatchingGame.IsInMatchingGame() or MatchingGame.IsInPVPGame() then
	
		-- What rating did we have to send (bg, arena ?)
		local matchType = 3
		local myUnit = GameLib:GetPlayerUnit()
		local myRating = MatchingGame.GetPvpRating(matchType).nRating
		local myWins = MatchingGame.GetPvpRating(matchType).nWins
		local myLosses = MatchingGame.GetPvpRating(matchType).nLosses
		local strMsg = myRating.."."..myWins.."."..myLosses
		
		-- Send it !
		self.ratingShare:SendMessage(strMsg)
		
		-- Trigger rating received for my own character!
		self:OnRatingReceived(self.ratingShareChannel, strMsg, myUnit:GetName())
		
	end
end

-- Update PVPTab
function PVPTab:Update()

	-- forEach team
	for key, team in pairs(self.liveStatistics.arTeamStats) do
		-- Define team container
		self:DefineTeamContainers(team.strTeamName, team.bIsMyTeam)
		self:UpdateTeamStats(team)
	end
	
	if self.ratingShare and self.enableRS then
		-- Send my rating to others
		self:SendMyRating()
	end
		
	-- ForEach PlayerInfo
	for key, playerInfo in pairs(self.liveStatistics.arParticipantStats) do
		
		-- If player is not in loading
		if playerInfo.strName ~= Apollo.GetString("PlayerNameLookupPending") then
		
			-- Player has his playerContainer ?
			if self.players[playerInfo.strName] then
				self:UpdatePlayerContainer(playerInfo)
				
				if self.players[playerInfo.strName].enemy then
					-- Update enemy role (relative to actual stats)
					self:DefineEnemyRole(self.players[playerInfo.strName])
				else
					-- Update teamMate role (one can change his stuff)
					if self.players[playerInfo.strName].unitAttached then
						self:DefineTeamMateRole(self.players[playerInfo.strName])
					end
				end
			else
				self:MakePlayerContainer(playerInfo)
			end
		end
	end
	
	-- Determine badge
	self:DefinePlayersBadges()
end

-- Define players badges, no more :d
function PVPTab:DefinePlayersBadges()
	
	-- Determine most values
	local MostDPS = 0
	local MostHeal = 0
	local MostKills = 0
	local MostAssists = 0
	local MostDPSReceived = 0
	local MostHealReceived = 0
	
	local MostDPSEnemy = 0
	local MostHealEnemy = 0
	local MostKillsEnemy = 0
	local MostAssistsEnemy = 0
	local MostDPSReceivedEnemy = 0
	local MostHealReceivedEnemy = 0
	
	-- forEach player
	for key, player in pairs(self.players) do
		
		if player.enemy then
			if player.statsDefined then
				if player.afk == false then
		
					if player.damage > MostDPSEnemy then 
						MostDPSEnemy = player.damage
					end
					
					if player.heal > MostHealEnemy then
						MostHealEnemy = player.heal
					end
					
					if player.kills > MostKillsEnemy then
						MostKillsEnemy = player.kills
					end
		
					if player.assists > MostAssistsEnemy then
						MostAssistsEnemy = player.assists
					end
					
					if player.damageReceived > MostDPSReceivedEnemy then
						MostDPSReceivedEnemy = player.damageReceived
					end
					
					if player.healReceived > MostHealReceivedEnemy then
						MostHealReceivedEnemy = player.healReceived
					end
				end
			end
		else
			if player.statsDefined then	
				if player.afk == false then
		
					if player.damage > MostDPS then 
						MostDPS = player.damage
					end
					
					if player.heal > MostHeal then
						MostHeal = player.heal
					end
					
					if player.kills > MostKills then
						MostKills = player.kills
					end
		
					if player.assists > MostAssists then
						MostAssists = player.assists
					end
					
					if player.damageReceived > MostDPSReceived then
						MostDPSReceived = player.damageReceived
					end
					
					if player.healReceived > MostHealReceived then
						MostHealReceived = player.healReceived
					end
				end
			end
		end

	end

	-- Define badge for players who has most values
	for key, player in pairs(self.players) do
	
		if player.enemy then
		
			-- Is first DPS
			if MostDPSEnemy > 5000 then
				if player.damage == MostDPSEnemy then
					local badge = Apollo.LoadForm(self.xmlDoc, "firstDPS", player.interface:FindChild("playerBadges"), self)
					badge:SetData(6)
				end
			end
			
			-- Is first healer
			if MostHealEnemy > 5000 then
				if player.heal == MostHealEnemy then
					local badge = Apollo.LoadForm(self.xmlDoc, "firstHeal", player.interface:FindChild("playerBadges"), self)
					badge:SetData(5)
				end
			end
			
			-- has most kills
			if MostKillsEnemy > 1 then
				if player.kills == MostKillsEnemy then
					local badge = Apollo.LoadForm(self.xmlDoc, "mostKills", player.interface:FindChild("playerBadges"), self)
					badge:SetData(4)
				end		
			end
			
			-- has most assists
			if MostAssistsEnemy > 1 then
				if player.assists == MostAssistsEnemy then
					local badge = Apollo.LoadForm(self.xmlDoc, "mostAssists", player.interface:FindChild("playerBadges"), self)
					badge:SetData(3)
				end
			end
			
			-- is most healed
			if MostHealReceivedEnemy > 5000 then
				if player.healReceived == MostHealReceivedEnemy then
					local badge = Apollo.LoadForm(self.xmlDoc, "mostHealed", player.interface:FindChild("playerBadges"), self)
					badge:SetData(1)
				end
			end
			
			-- is most tank
			if MostDPSReceivedEnemy > 5000 then
				if player.damageReceived == MostDPSReceivedEnemy then
					local badge = Apollo.LoadForm(self.xmlDoc, "mostTanked", player.interface:FindChild("playerBadges"), self)
					badge:SetData(2)
				end
			end
			
		else 

			-- Is first DPS
			if MostDPS > 5000 then
				if player.damage == MostDPS then
					local badge = Apollo.LoadForm(self.xmlDoc, "firstDPS", player.interface:FindChild("playerBadges"), self)
					badge:SetData(6)
				end
			end
			
			-- Is first healer
			if MostHeal > 5000 then
				if player.heal == MostHeal then
					local badge = Apollo.LoadForm(self.xmlDoc, "firstHeal", player.interface:FindChild("playerBadges"), self)
					badge:SetData(5)
				end
			end
			
			-- has most kills
			if MostKills > 1 then
				if player.kills == MostKills then
					local badge = Apollo.LoadForm(self.xmlDoc, "mostKills", player.interface:FindChild("playerBadges"), self)
					badge:SetData(4)
				end		
			end
			
			-- has most assists
			if MostAssists > 1 then
				if player.assists == MostAssists then
					local badge = Apollo.LoadForm(self.xmlDoc, "mostAssists", player.interface:FindChild("playerBadges"), self)
					badge:SetData(3)
				end
			end
			
			-- is most healed
			if MostHealReceived > 5000 then
				if player.healReceived == MostHealReceived then
					local badge = Apollo.LoadForm(self.xmlDoc, "mostHealed", player.interface:FindChild("playerBadges"), self)
					badge:SetData(1)
				end
			end
			
			-- is most tank
			if MostDPSReceived > 5000 then
				if player.damageReceived == MostDPSReceived then
					local badge = Apollo.LoadForm(self.xmlDoc, "mostTanked", player.interface:FindChild("playerBadges"), self)
					badge:SetData(2)
				end
			end
		end
		
		-- Do we have his rating ?
		if player.rating then
			local badge = Apollo.LoadForm(self.xmlDoc, "hasRating", player.interface:FindChild("playerBadges"), self)
			badge:SetTooltip("This player use PVPTab!\nRating : "..player.rating.."\nWins : "..player.wins.."\nLosses : "..player.losses)
			badge:SetData(0)
		end
				
		-- Order players badges
		player.interface:FindChild("playerBadges"):ArrangeChildrenHorz(0, function(a,b) return a:GetData() > b:GetData() end)

	end
end

-- Round a number
function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

-- Define role for a team mate (when have his stats, its easy)
function PVPTab:DefineTeamMateRole(player)
	local playerUnit = player.unit
	local playerName = playerUnit:GetName()
	
	if GameLib:GetPlayerUnit() then
		local myName = GameLib:GetPlayerUnit():GetName()
		local AP = playerUnit:GetAssaultPower() or 0
		local SP = playerUnit:GetSupportPower() or 0
		
		-- Define his role
		if playerName == myName then
			-- My own character is pink bordered
			player.interface:FindChild("playerColor"):SetSprite("CRB_NameplateSprites:sprNp_HealthBarHostile")
		else
		
			local classId = playerUnit:GetClassId()
			local roleDetermined = false
	
			-- Detect Healer
			if classId == GameLib.CodeEnumClass.Esper or classId == GameLib.CodeEnumClass.Medic or classId == GameLib.CodeEnumClass.Spellslinger then
				if SP > AP then
					player.interface:FindChild("playerColor"):SetSprite("CRB_NameplateSprites:sprNp_HealthBarFriendly")
					roleDetermined = true
				end
			end
				
			-- Detect Tank
			if classId == GameLib.CodeEnumClass.Stalker or classId == GameLib.CodeEnumClass.Warrior or classId == GameLib.CodeEnumClass.Engineer then
				if SP > AP and not roleDetermined then
					player.interface:FindChild("playerColor"):SetSprite("CRB_NameplateSprites:sprNp_HealthBarBlue")
					roleDetermined = true
				end
			end
			
			-- Detect DPS
			if not roleDetermined then
				player.interface:FindChild("playerColor"):SetSprite("CRB_NameplateSprites:sprNp_HealthBarNeutral")
			end
		
		end
	end
end

-- Define enemy role (its relative to his matchinggame stats, we need stats then. We dont have access to his ap, sp)
function PVPTab:DefineEnemyRole(player)
	local classId = player.classId
	local roleDetermined = false
	local damage = player.damage
	local heal = player.heal

	-- If no stats
	if heal == 0 and damage == 0 then
		-- Nothing for the moment
	else
		-- Detect Healer
		if classId == GameLib.CodeEnumClass.Esper or classId == GameLib.CodeEnumClass.Medic or classId == GameLib.CodeEnumClass.Spellslinger then
			if heal > damage then
				player.interface:FindChild("playerColor"):SetSprite("CRB_NameplateSprites:sprNp_HealthBarFriendly")
				roleDetermined = true
			end
		end
			
		-- Detect Tank [???, cant define it]
		if classId == GameLib.CodeEnumClass.Stalker or classId == GameLib.CodeEnumClass.Warrior or classId == GameLib.CodeEnumClass.Engineer then
			player.interface:FindChild("playerColor"):SetSprite("CRB_NameplateSprites:sprNp_HealthBarBlue")
		end
		
		-- Detect DPS
		if not roleDetermined then
			player.interface:FindChild("playerColor"):SetSprite("CRB_NameplateSprites:sprNp_HealthBarNeutral")
		end
	end
end

---------------------------------------------------------------------------------------------------
-- PVPTabForm Functions
---------------------------------------------------------------------------------------------------

function PVPTab:OpenOrderList( wndHandler, wndControl, eMouseButton )
	if self.wndOrdering:IsVisible() then
		self.wndOrdering:Show(false, true)
	else 
		self.wndSettings:Close()
		self.wndOrdering:Show(true, false)
		self.wndOrdering:ToFront()
	end
end

function PVPTab:OpenSettings()
	if self.wndSettings:IsVisible() then
		self.wndSettings:Close()
	else
		self:LoadSettingsUI()
		self.wndOrdering:Close()
		self.wndSettings:Invoke()
		self.wndSettings:ToFront()
	end	
end

function PVPTab:OrderByDPS( wndHandler, wndControl, eMouseButton )
	self:OrderPlayers(1)
	self.orderId = 1
	self.wndOrdering:Show(false, true)
	self.wndMain:FindChild("OrderBtn"):SetText("Damage dealt")
end

function PVPTab:OrderByHEAL( wndHandler, wndControl, eMouseButton )
	self:OrderPlayers(2)
	self.orderId = 2
	self.wndOrdering:Show(false, true)
	self.wndMain:FindChild("OrderBtn"):SetText("Heal done")
end

function PVPTab:OrderByKILLS( wndHandler, wndControl, eMouseButton )
	self:OrderPlayers(3)
	self.orderId = 3
	self.wndOrdering:Show(false, true)
	self.wndMain:FindChild("OrderBtn"):SetText("Most kills")
end

function PVPTab:OrderByDEATHS( wndHandler, wndControl, eMouseButton )
	self:OrderPlayers(4)
	self.orderId = 4
	self.wndOrdering:Show(false, true)
	self.wndMain:FindChild("OrderBtn"):SetText("Most deaths")
end

function PVPTab:OrderByASSISTS( wndHandler, wndControl, eMouseButton )
	self:OrderPlayers(5)
	self.orderId = 5
	self.wndOrdering:Show(false, true)
	self.wndMain:FindChild("OrderBtn"):SetText("Most assists")
end

function PVPTab:closeTab()
	self.wndMain:Show(false, true)
	self.wndOrdering:Show(false, true)
end

function PVPTab:toggleRS( wndHandler, wndControl, eMouseButton )
	self:SetSettingsUI()
end

function PVPTab:toggleAnimatedFace( wndHandler, wndControl, eMouseButton )
	self:SetSettingsUI()
end

-----------------------------------------------------------------------------------------------
-- PVPTab Instance
-----------------------------------------------------------------------------------------------
local PVPTabInst = PVPTab:new()
PVPTabInst:Init()
